using Unity;
using ProjectRepairWork.Repositories;
using ProjectRepairWork.Repositories.Implementation;
using Unity.Lifetime;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using Serilog;
using Unity.Microsoft.Logging;
namespace ProjectRepairWork
{
    internal static class Program
    {
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            Application.Run(CreateContainer().Resolve<FormRepairWork>());
        }
        private static IUnityContainer CreateContainer()
        {
            var container = new UnityContainer()
    .AddExtension(new Diagnostic());

            container.AddExtension(new LoggingExtension(CreateLoggerFactory()));

            container.RegisterType<ICostRepository, CostRepository>();
            container.RegisterType<IContractorsRepository, ContractorsRepository>();
            container.RegisterType<IPremisesRepository, PremisesRepository>();
            container.RegisterType<IRepairRepository, RepairRepository>();
            container.RegisterType<IWorksRepository, WorksRepository>();
            container.RegisterType<IConnectionString, ConnectionString>();
            container.RegisterType<IBuildingMaterialsRepository, BuildingMaterialsRepository>();
            container.RegisterType<IMaterialProcurementRepository, MaterialProcurementRepository>();
            return container;
        }

        private static LoggerFactory CreateLoggerFactory()
        {
            var loggerFactory = new LoggerFactory();
            loggerFactory.AddSerilog(new LoggerConfiguration()
                .ReadFrom.Configuration(new ConfigurationBuilder()
                .SetBasePath(Directory.GetCurrentDirectory())
                .AddJsonFile("appsettings.json")
                .Build())
                .CreateLogger());
            return loggerFactory;
        }

    }
}