﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectRepairWork.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectRepairWork.Repositories.Implementation;

public class MaterialProcurementRepository : IMaterialProcurementRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<MaterialProcurementRepository> _logger;

    public MaterialProcurementRepository(IConnectionString connectionString, ILogger<MaterialProcurementRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreateMaterialProcurement(MaterialProcurement materialProcurement)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(materialProcurement));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"
                INSERT INTO MaterialProcurement (MaterialsId, MaterialsPrice, ProcurmentDate)
                VALUES (@MaterialsId, @MaterialsPrice, @ProcurmentDate)";
            connection.Execute(queryInsert, materialProcurement);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }

    public MaterialProcurement ReadMaterialProcurementById(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new
            NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"
                SELECT * FROM MaterialProcurement
                WHERE Id=@id";
            var materialProcurement = connection.QueryFirst<MaterialProcurement>(querySelect, new
            {
                id
            });
            _logger.LogDebug("Найденный объект: {json}",
            JsonConvert.SerializeObject(materialProcurement));
            return materialProcurement;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }
    public IEnumerable<MaterialProcurement> ReadMaterialProcurement()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new
            NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = "SELECT * FROM MaterialProcurement";
            var materialProcurement = connection.Query<MaterialProcurement>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}",
            JsonConvert.SerializeObject(materialProcurement));
            return materialProcurement;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }

}
