﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectRepairWork.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectRepairWork.Repositories.Implementation;

public class RepairRepository : IRepairRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<RepairRepository> _logger;

    public RepairRepository(IConnectionString connectionString, ILogger<RepairRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CraeteRepair(Repair repair)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}",
        JsonConvert.SerializeObject(repair));
        try
        {
            using var connection = new
            NpgsqlConnection(_connectionString.ConnectionString);
            connection.Open();
            using var transaction = connection.BeginTransaction();
            var queryInsert = @"
                INSERT INTO Repair (ContractorsId, PremisesId, DateRepair)
                VALUES (@ContractorsId, @PremisesId, @DateRepair);
                SELECT MAX(Id) FROM Repair";
            var repairId =
            connection.QueryFirst<int>(queryInsert, repair, transaction);
            var querySubInsert = @"
                INSERT INTO RepairRepair (RepairId, WorkId, Count)
                VALUES (@RepairId, @WorkId, @Count)";
            foreach (var elem in repair.RepairRepair)
            {
                connection.Execute(querySubInsert, new
                {
                    repairId,
                    elem.WorkId,
                    elem.Count
                }, transaction);
            }
            transaction.Commit();
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }

    public IEnumerable<Repair> ReadRepair(int? contractorsId = null, int? costId = null, int? worksId = null, int? premisesId = null)
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = "SELECT * FROM Repair";
            var repair = connection.Query<Repair>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}",
            JsonConvert.SerializeObject(repair));
            return repair;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }

    public IEnumerable<RepairRepair> ReadRepairRepair()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new
            NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = "SELECT * FROM RepairRepair";
            var repairRepairs = connection.Query<RepairRepair>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}",
            JsonConvert.SerializeObject(repairRepairs));
            return repairRepairs;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }
}
